/***************************************************************************
 *   Copyright (C) 2004 by Tyler Montbriand                                *
 *   tsm@accesscomm.ca                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 *  Test driver for SDL_AudioIn:  random noise.
 *  Tyler Montbriand, 2004
 */

#include "audio_detect.h"

#ifdef AUDIOIN_RAND

#include <stdlib.h>
#include <SDL/SDL_audio.h>
#include <SDL/SDL_thread.h>
#include "SDL_audioin.h"
#include "SDL_audioin_internal.h"

static int  RAND_init();
static void RAND_quit();
static int  RAND_open(SDL_AudioSpec *desired, SDL_AudioSpec *result);
static void RAND_close();
static int  RAND_pause(int pause_on);

Sound_InputFunctions driver_RAND=
{
  "440-Hz tuning fork null input driver\n",
  "tsm@accesscomm.ca",
  null_init,
  null_quit,
  RAND_open,
  RAND_close,
  RAND_pause  
};

static SDL_AudioSpec ref;
static SDL_Thread   *thread=NULL;

static int RAND_thread(void *dat)
{
  UniPtr ptr;
  int n;
  int ticks=0;

  if(audioin_status==SDL_AUDIO_STOPPED)
    return(-1);

  ptr.vptr=malloc(ref.size);
  if(ptr.vptr==NULL)
    return(-1);

  ticks=AudioSpecTicks(&ref);
  if(ticks<10) ticks=10;

  while(audioin_status!=SDL_AUDIO_STOPPED)
  {
    if(audioin_status==SDL_AUDIO_PLAYING)
    {
      for(n=0; n<ref.size; n++)
        ptr.u8ptr[n]=rand()%256;

      if(ref.callback!=NULL)
        ref.callback(ref.userdata,ptr.u8ptr,ref.size);
    }
    else /* paused */
    {
    }
    SDL_Delay(ticks);
  }

  free(ptr.vptr);

  return(0);  
}

static int RAND_open(SDL_AudioSpec *desired, SDL_AudioSpec *result)
{
  if(audioin_status!=SDL_AUDIO_STOPPED)
  {
    SDL_SetError("RAND driver already running");
    return(-1);
  }

  if(!AudioSpecValid(desired))
    return(-1);

  ref=(*desired);  
  ref.size=AudioSpecSize(desired);

  audioin_status=SDL_AUDIO_PAUSED;
  thread=SDL_CreateThread(RAND_thread,NULL);
  if(thread==NULL)
  {
    audioin_status=SDL_AUDIO_STOPPED;
    SDL_SetError("Couldn't create RAND thread\n");
    return(-1);
  }

  if(result!=NULL)
    (*result)=ref;

  return(0);
}

static void RAND_close(void)
{
  if(thread!=NULL)
  {
    int status;

    if(audioin_status!=SDL_AUDIO_STOPPED)
      audioin_status=SDL_AUDIO_STOPPED;

    SDL_WaitThread(thread,&status);
    thread=NULL;

    if(status!=0)
      fprintf(stderr,"Warning, RAND thread exited with status %d\n",status);
  }  
}

static int  RAND_pause(int pause_on)
{
  if(audioin_status!=SDL_AUDIO_STOPPED)
  {
    if(pause_on)
      audioin_status=SDL_AUDIO_PAUSED;
    else
      audioin_status=SDL_AUDIO_PLAYING;
  }
  return(0);
}

#endif/*AUDIOIN_RAND*/
